[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**cigarillo** is an R/Bioconductor package for efficient manipulation of CIGAR strings.

See <https://bioconductor.org/packages/cigarillo> for more information, including how to install the release version of the package (please refrain from installing directly from GitHub).

